from os import environ


class Keys(object):
    """
    This encapsulates a Paystack account keys.

    By default, `Keys` will look for the following environment variables to
    set its arguments if you do not supply any arguments when instantiating
    it:
    -	`secret_key`: `PAYSTACK_SECRET_KEY`
    -	`public_key`: `PAYSTACK_PUBLIC_KEY`
    -	`callback_url`: `PAYSTACK_CALLBACK_URL`

    If you have the same information saved in an environment variable with a
    different name from what `Keys` is looking for, you can supply your
    environment variable name as the argument. e.g
    >>> Keys(secret_key='MY_OWN_ENVIRONMENT_VARIABLE_NAME', public_key='OTHER_ENVIRONMENT_VARIABLE')
    >>>

    Finally, you can just manually enter your Paystack keys. e.g
    >>> Keys(secret_key='sc_domain_topSEKrit', public_key='pk_domain_Ud0NtKNo')
    >>>
    """

    def __init__(self, secret_key='', public_key='', callback_url=''):
        """
        By default, the parameters are set as follows:
        -	`secret_key`: `PAYSTACK_SECRET_KEY`
        -	`public_key`: `PAYSTACK_PUBLIC_KEY`
        -	`callback_url`: `PAYSTACK_CALLBACK_URL`

        You can supply your own matching environment variable names of just
        matching key.

        :param secret_key: String environment variable name containing Paystack
        secret key or Paystack secret key
        :param public_key: String environment variable name containing Paystack
        public key or Paystack public key
        :param callback_url: String environment variable name containing
        Paystack callback url or Paystack callback url
        """
        self._secret_key = environ.get(secret_key, secret_key)
        self._public_key = environ.get(public_key, public_key)
        self._callback_url = environ.get(callback_url, callback_url)

    @property
    def secret_key(self):
        return self._secret_key

    @property
    def public_key(self):
        return self._public_key

    @property
    def callback_url(self):
        return self._callback_url




from unittest import TestCase

import time

from utils import build_params
from api import TransferControl, PaymentPage, Customer, Invoice

SECRET_KEY = 'sk_test_04d09de6922fa975afdc7b5eb40ff357e74b9f01'
PUBLIC_KEY = 'pk_test_666fbd4fca373a2a88ec13f6f22dfb46cbdec709'


class TestCustomer(TestCase):
	def setUp(self):
		self.api = Customer(secret_key=SECRET_KEY, public_key=PUBLIC_KEY)
		self.api.create_customer(email='test@example.com')
		self.customer_id = self.api.customer_id
		self.customer_code = self.api.customer_code

	def test_create_customer(self):
		self.assertEqual(self.api.ctx.status, True)

	def test_fetch_customer(self):
		self.api.fetch_customer(email_or_id_or_customer_code='test@example.com')
		self.assertEqual(self.customer_code, self.api.customer_code)
		self.assertEqual(self.customer_id, self.api.customer_id)
		self.assertEqual(True, self.api.ctx.status)

		self.api.fetch_customer(email_or_id_or_customer_code='not@example.com')
		self.assertEqual(False, self.api.ctx.status)
		self.assertEqual(self.api.customer_id, '')
		self.assertEqual(self.api.customer_code, '')


class TestInvoice(TestCase):
	def setUp(self):
		self.api = Customer(secret_key=SECRET_KEY, public_key=PUBLIC_KEY)
		self.api.create_customer(email='test@example.com')
		self.customer_code = self.api.customer_code

	def test_create_invoice(self):
		api = Invoice(secret_key=SECRET_KEY, public_key=PUBLIC_KEY)
		api.create_invoice(customer=self.customer_code, amount=500000, due_date='2020-12-20')

		self.assertEqual(self.api.ctx.status, True)

	def test_list_invoices(self):
		api = Invoice(secret_key=SECRET_KEY, public_key=PUBLIC_KEY)
		api.list_invoices()

		self.assertEqual(api.ctx.status, True)


class TestTransferControl(TestCase):
	def test_balance(self):
		api = TransferControl(secret_key=SECRET_KEY, public_key=PUBLIC_KEY)
		api.get_balance()

		self.assertEqual(api.ctx.status, True)
		self.assertEqual(api.ctx.message, 'Balances retrieved')


class TestPaymentPage(TestCase):
	def test_create_page(self):
		r = PaymentPage(secret_key=SECRET_KEY, public_key=PUBLIC_KEY)
		slug = str(time.time())
		r.create_page(name='test page', slug=slug)

		self.assertEqual(r.ctx.status, True)
		self.assertEqual(r.ctx.message, 'Page created')
		self.assertEqual(r.name, 'test page')
		self.assertEqual(r.slug, slug)
		self.assertEqual(r.page_url, 'https://paystack.com/pay/{slug}'.format(slug=slug))


class TestFunctions(TestCase):
	def test_build_params(self):
		self.assertEqual(build_params(one=1, two=2), {'one': 1, 'two': 2})
		self.assertEqual(build_params(), {})
		self.assertEqual(build_params(a='a', b=None, c='b'), {'a': 'a', 'c': 'b'})



from .request import PaystackRequest
from .utils import build_params


class Customer(object):
    def __init__(self, **kwargs):
        self.__base = PaystackRequest(**kwargs)
        self.__url = 'https://api.paystack.co/customer'

    @property
    def ctx(self):
        return self.__base

    @property
    def url(self):
        return self.__url

    @url.setter
    def url(self, value):
        self.__url = value

    @property
    def customer_code(self):
        return self.ctx.data.get('customer_code', '')

    @property
    def customer_id(self):
        return self.ctx.data.get('customer_id', '')

    def __getattr__(self, item):
        return getattr(self.__base, item)

    def create_customer(self, email, first_name=None, last_name=None, phone=None, metadata=None):
        params = build_params(email=email, first_name=first_name,
                              last_name=last_name, phone=phone,
                              metadata=metadata)

        self.ctx.post(self.url, json=params)

    def fetch_customer(self, email_or_id_or_customer_code):
        """
        If there is no customer that satisfies the `email_or_id_or_customer_code`
        argument, it returns None

        :param email_or_id_or_customer_code: Customer email or customer id or customer code
        :return: dict
        """
        url_ = '{url}/{id}'.format(url=self.url, id=email_or_id_or_customer_code)
        self.ctx.get(url_)


"""
Use this to create paystack invoices
Note: `__url` is deprecated. Use either `__notification_url` or `__payment_url`
depending on your use case
"""
class Invoice(object):
    def __init__(self, **kwargs):
        self.__base = PaystackRequest(**kwargs)
        self.__url = 'https://api.paystack.co/paymentrequest'   # deprecated
        self.__notification_url = 'https://api.paystack.co/paymentrequest'
        self.__payment_url = 'https://paystack.com/pay'

    @property
    def ctx(self):
        return self.__base

    @property
    def url(self):  # deprecated
        return self.__url

    @url.setter # deprecated
    def url(self, value):
        self.__url = value

    @property
    def payment_url(self):
        return self.__notification_url + self.request_code

    @property
    def notification_url(self):
        return self.__notification_url + self.request_code

    @property
    def invoice_code(self):
        # do not use this any more
        return None

    @property
    def invoice_number(self):
        return self.ctx.data.get('invoice_number')

    @property
    def invoice_id(self):
        return None

    @property
    def request_code(self):
        return self.ctx.data.get('request_code')

    def __getattr__(self, item):
        return getattr(self.__base, item)

    def create_invoice(self, customer, amount, due_date, description=None,
                       line_items=None, tax=None, currency='NGN',
                       metadata=None, send_notification=True, draft=False,
                       has_invoice=False, invoice_number=None):

        # Note that `invoice_number` has to be an integer else it won't have
        # effect
        params = build_params(customer=customer, amount=amount,
                              due_date=due_date, description=description,
                              line_items=line_items, tax=tax, currency=currency,
                              metadata=metadata,
                              send_notification=send_notification, draft=draft,
                              has_invoice=has_invoice, invoice_number=invoice_number)

        self.ctx.post(self.url, json=params)

    def list_invoices(self, customer=None, paid=None, status=None, currency=None, include_archive=None):
        params = build_params(
            customer=customer, paid=paid, status=status, currency=currency, include_archive=include_archive)
        self.ctx.get(self.url, payload=params)


class TransferControl(object):
    def __init__(self, **kwargs):
        self.__base = PaystackRequest(**kwargs)

    def __getattr__(self, item):
        return getattr(self.__base, item)

    @property
    def ctx(self):
        return self.__base

    def get_balance(self):
        url = '{host}/balance'.format(host=self.api_url)
        self.ctx.get(url)


class PaymentPage(object):
    def __init__(self, **kwargs):
        self.__base = PaystackRequest(**kwargs)
        self._page_url = None
        self.__paystack_payment_url = 'https://paystack.com/pay'

    def __getattr__(self, item):
        return getattr(self.__base, item)

    @property
    def ctx(self):
        return self.__base

    @property
    def paystack_payment_url(self):
        return self.__paystack_payment_url

    @property
    def slug(self):
        return self.ctx.data.get('slug')

    @property
    def page_url(self):
        if self.ctx.status:
            page_url = '{http}/{slug}'.format(http=self.paystack_payment_url, slug=self.slug)
            return page_url

    @property
    def name(self):
        return self.ctx.data.get('name')

    def create_page(self, name, description=None, amount=None, slug=None, redirect_url=None, custom_fields=None):
        params = build_params(
            name=name, description=description, amount=amount, slug=slug,
            redirect_url=redirect_url, custom_fields=custom_fields)

        url = '{host}/page'.format(host=self.api_url)
        self.ctx.post(url, json=params)



from setuptools import setup

setup(
	name='paystakk',
	version='0.0.4',
	packages=['paystakk'],
	url='',
	license='MIT',
	author='tundebabzy',
	author_email='tundebabzy@gmail.com',
	description='Unofficial Paystacks API wrapper'
)
